@@ -8,7 +8,7 @@
 
 /**
  * @file
- * @brief Server client declarations
+ * @brief Client Declarations
  */
 
 #ifndef __GEARMAN_SERVER_CLIENT_H__
@@ -19,10 +19,28 @@ extern "C" {
 #endif
 
 /**
- * @addtogroup gearman_server_client Server Client Handling
+ * @ingroup gearman_server_client
+ */
+struct gearman_server_client_st
+{
+  struct {
+    bool allocated;
+  } options;
+  gearman_server_con_st *con;
+  gearman_server_client_st *con_next;
+  gearman_server_client_st *con_prev;
+  gearman_server_job_st *job;
+  gearman_server_client_st *job_next;
+  gearman_server_client_st *job_prev;
+};
+
+/**
+ * @addtogroup gearman_server_client Client Declarations
  * @ingroup gearman_server
+ *
  * This is a low level interface for gearman server clients. This is used
  * internally by the server interface, so you probably want to look there first.
+ *
  * @{
  */
 
