@@ -37,7 +37,7 @@ static gearman_return_t _con_add(gearmand_thread_st *thread,
 
 gearman_return_t gearmand_con_create(gearmand_st *gearmand, int fd,
                                      const char *host, const char *port,
-                                     gearman_con_add_fn *add_fn)
+                                     gearman_connection_add_fn *add_fn)
 {
   gearmand_con_st *dcon;
   gearmand_con_st *free_dcon_list;
@@ -54,7 +54,7 @@ gearman_return_t gearmand_con_create(gearmand_st *gearmand, int fd,
     if (dcon == NULL)
     {
       close(fd);
-      GEARMAN_FATAL(gearmand, "gearmand_con_create:malloc")
+      gearmand_log_fatal(gearmand, "gearmand_con_create:malloc");
       return GEARMAN_MEMORY_ALLOCATION_FAILURE;
     }
   }
@@ -126,13 +126,15 @@ gearman_return_t gearmand_con_create(gearmand_st *gearmand, int fd,
 
 void gearmand_con_free(gearmand_con_st *dcon)
 {
-  assert(event_del(&(dcon->event)) == 0);
+  int del_ret= event_del(&(dcon->event));  
+  assert(del_ret == 0);
 
   /* This gets around a libevent bug when both POLLIN and POLLOUT are set. */
   event_set(&(dcon->event), dcon->fd, EV_READ, _con_ready, dcon);
   event_base_set(dcon->thread->base, &(dcon->event));
   event_add(&(dcon->event), NULL);
-  assert(event_del(&(dcon->event)) == 0);
+  del_ret= event_del(&(dcon->event));
+  assert(del_ret == 0);
 
   gearman_server_con_free(dcon->server_con);
   GEARMAN_LIST_DEL(dcon->thread->dcon, dcon,)
@@ -177,14 +179,14 @@ void gearmand_con_check_queue(gearmand_thread_st *thread)
   }
 }
 
-gearman_return_t gearmand_con_watch(gearman_con_st *con, short events,
-                                    void *arg __attribute__ ((unused)))
+gearman_return_t gearmand_connection_watch(gearman_connection_st *con, short events,
+                                    void *context __attribute__ ((unused)))
 {
-  (void) arg;
+  (void) context;
   gearmand_con_st *dcon;
   short set_events= 0;
 
-  dcon= (gearmand_con_st *)gearman_con_context(con);
+  dcon= (gearmand_con_st *)gearman_connection_context(con);
   dcon->con= con;
 
   if (events & POLLIN)
@@ -195,24 +197,27 @@ gearman_return_t gearmand_con_watch(gearman_con_st *con, short events,
   if (dcon->last_events != set_events)
   {
     if (dcon->last_events != 0)
-      assert(event_del(&(dcon->event)) == 0);
+    {
+      int del_ret= event_del(&(dcon->event));
+      assert(del_ret == 0);
+    }
     event_set(&(dcon->event), dcon->fd, set_events | EV_PERSIST, _con_ready,
               dcon);
     event_base_set(dcon->thread->base, &(dcon->event));
 
     if (event_add(&(dcon->event), NULL) == -1)
     {
-      GEARMAN_FATAL(dcon->thread->gearmand, "_con_watch:event_add:-1")
+      gearmand_log_fatal(dcon->thread->gearmand, "_con_watch:event_add:-1");
       return GEARMAN_EVENT;
     }
 
     dcon->last_events= set_events;
   }
 
-  GEARMAN_CRAZY(dcon->thread->gearmand, "[%4u] %15s:%5s Watching  %6s %s",
-                dcon->thread->count, dcon->host, dcon->port,
-                events & POLLIN ? "POLLIN" : "",
-                events & POLLOUT ? "POLLOUT" : "")
+  gearmand_log_crazy(dcon->thread->gearmand, "[%4u] %15s:%5s Watching  %6s %s",
+                     dcon->thread->count, dcon->host, dcon->port,
+                     events & POLLIN ? "POLLIN" : "",
+                     events & POLLOUT ? "POLLOUT" : "");
 
   return GEARMAN_SUCCESS;
 }
@@ -233,17 +238,17 @@ static void _con_ready(int fd __attribute__ ((unused)), short events,
   if (events & EV_WRITE)
     revents|= POLLOUT;
 
-  ret= gearman_con_set_revents(dcon->con, revents);
+  ret= gearman_connection_set_revents(dcon->con, revents);
   if (ret != GEARMAN_SUCCESS)
   {
     gearmand_con_free(dcon);
     return;
   }
 
-  GEARMAN_CRAZY(dcon->thread->gearmand, "[%4u] %15s:%5s Ready     %6s %s",
-                dcon->thread->count, dcon->host, dcon->port,
-                revents & POLLIN ? "POLLIN" : "",
-                revents & POLLOUT ? "POLLOUT" : "")
+  gearmand_log_crazy(dcon->thread->gearmand, "[%4u] %15s:%5s Ready     %6s %s",
+                     dcon->thread->count, dcon->host, dcon->port,
+                     revents & POLLIN ? "POLLIN" : "",
+                     revents & POLLOUT ? "POLLOUT" : "");
 
   gearmand_thread_run(dcon->thread);
 }
@@ -277,8 +282,7 @@ static gearman_return_t _con_add(gearmand_thread_st *thread,
     }
   }
 
-  GEARMAN_INFO(thread->gearmand, "[%4u] %15s:%5s Connected", thread->count,
-               dcon->host, dcon->port)
+  gearmand_log_info(thread->gearmand, "[%4u] %15s:%5s Connected", thread->count, dcon->host, dcon->port);
 
   GEARMAN_LIST_ADD(thread->dcon, dcon,)
 
