@@ -74,16 +74,20 @@ gearman_server_st *gearman_server_create(gearman_server_st *server)
     if (server == NULL)
       return NULL;
 
-    server->options= GEARMAN_SERVER_ALLOCATED;
+    server->options.allocated= true;
   }
   else
-    server->options= 0;
+    server->options.allocated= false;
 
+  server->state.queue_startup= false;
+  server->flags.round_robin= false;
+  server->flags.threaded= false;
   server->shutdown= false;
   server->shutdown_graceful= false;
   server->proc_wakeup= false;
   server->proc_shutdown= false;
   server->job_retries= 0;
+  server->worker_wakeup= 0;
   server->thread_count= 0;
   server->free_packet_count= 0;
   server->function_count= 0;
@@ -110,7 +114,7 @@ gearman_server_st *gearman_server_create(gearman_server_st *server)
   memset(server->unique_hash, 0,
          sizeof(gearman_server_job_st *) * GEARMAN_JOB_HASH_SIZE);
 
-  server->gearman= gearman_create(&(server->gearman_static));
+  server->gearman= gearman_universal_create(&(server->gearman_universal_static), NULL);
   if (server->gearman == NULL)
   {
     gearman_server_free(server);
@@ -179,9 +183,9 @@ void gearman_server_free(gearman_server_st *server)
   }
 
   if (server->gearman != NULL)
-    gearman_free(server->gearman);
+    gearman_universal_free(server->gearman);
 
-  if (server->options & GEARMAN_SERVER_ALLOCATED)
+  if (server->options.allocated)
     free(server);
 }
 
@@ -191,9 +195,15 @@ void gearman_server_set_job_retries(gearman_server_st *server,
   server->job_retries= job_retries;
 }
 
+void gearman_server_set_worker_wakeup(gearman_server_st *server,
+                                      uint8_t worker_wakeup)
+{
+  server->worker_wakeup= worker_wakeup;
+}
+
 void gearman_server_set_log_fn(gearman_server_st *server,
                                gearman_log_fn *function,
-                               const void *context, gearman_verbose_t verbose)
+                               void *context, gearman_verbose_t verbose)
 {
   server->log_fn= function;
   server->log_context= context;
@@ -230,7 +240,7 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
     if (ret != GEARMAN_SUCCESS)
       return ret;
 
-    packet->options&= (gearman_packet_options_t)~GEARMAN_PACKET_FREE_DATA;
+    packet->options.free_data= false;
 
     break;
 
@@ -277,7 +287,9 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
                                        packet->data_size, priority,
                                        server_client, &ret);
     if (ret == GEARMAN_SUCCESS)
-      packet->options&= (gearman_packet_options_t)~GEARMAN_PACKET_FREE_DATA;
+    {
+      packet->options.free_data= false;
+    }
     else if (ret == GEARMAN_JOB_QUEUE_FULL)
     {
       return _server_error_packet(server_con, "queue_full",
@@ -302,7 +314,7 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
     snprintf(job_handle, GEARMAN_JOB_HANDLE_SIZE, "%.*s",
              (uint32_t)(packet->arg_size[0]), (char *)(packet->arg[0]));
 
-    server_job= gearman_server_job_get(server_con->thread->server, job_handle);
+    server_job= gearman_server_job_get(server_con->thread->server, job_handle, NULL);
 
     /* Queue status result packet. */
     if (server_job == NULL)
@@ -461,7 +473,8 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
   case GEARMAN_COMMAND_WORK_DATA:
   case GEARMAN_COMMAND_WORK_WARNING:
     server_job= gearman_server_job_get(server_con->thread->server,
-                                       (char *)(packet->arg[0]));
+                                       (char *)(packet->arg[0]),
+                                       server_con);
     if (server_job == NULL)
     {
       return _server_error_packet(server_con, "job_not_found",
@@ -477,7 +490,8 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
 
   case GEARMAN_COMMAND_WORK_STATUS:
     server_job= gearman_server_job_get(server_con->thread->server,
-                                       (char *)(packet->arg[0]));
+                                       (char *)(packet->arg[0]),
+                                       server_con);
     if (server_job == NULL)
     {
       return _server_error_packet(server_con, "job_not_found",
@@ -511,7 +525,8 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
 
   case GEARMAN_COMMAND_WORK_COMPLETE:
     server_job= gearman_server_job_get(server_con->thread->server,
-                                       (char *)(packet->arg[0]));
+                                       (char *)(packet->arg[0]),
+                                       server_con);
     if (server_job == NULL)
     {
       return _server_error_packet(server_con, "job_not_found",
@@ -525,7 +540,7 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
       return ret;
 
     /* Remove from persistent queue if one exists. */
-    if (server_job->options & GEARMAN_SERVER_JOB_QUEUED &&
+    if (server_job->state & GEARMAN_SERVER_JOB_QUEUED &&
         server->queue_done_fn != NULL)
     {
       ret= (*(server->queue_done_fn))(server, (void *)server->queue_context,
@@ -543,7 +558,8 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
 
   case GEARMAN_COMMAND_WORK_EXCEPTION:
     server_job= gearman_server_job_get(server_con->thread->server,
-                                       (char *)(packet->arg[0]));
+                                       (char *)(packet->arg[0]),
+                                       server_con);
     if (server_job == NULL)
     {
       return _server_error_packet(server_con, "job_not_found",
@@ -562,7 +578,8 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
     snprintf(job_handle, GEARMAN_JOB_HANDLE_SIZE, "%.*s",
              (uint32_t)(packet->arg_size[0]), (char *)(packet->arg[0]));
 
-    server_job= gearman_server_job_get(server_con->thread->server, job_handle);
+    server_job= gearman_server_job_get(server_con->thread->server, job_handle,
+                                       server_con);
     if (server_job == NULL)
     {
       return _server_error_packet(server_con, "job_not_found",
@@ -583,7 +600,7 @@ gearman_return_t gearman_server_run_command(gearman_server_con_st *server_con,
     }
 
     /* Remove from persistent queue if one exists. */
-    if (server_job->options & GEARMAN_SERVER_JOB_QUEUED &&
+    if (server_job->state & GEARMAN_SERVER_JOB_QUEUED &&
         server->queue_done_fn != NULL)
     {
       ret= (*(server->queue_done_fn))(server, (void *)server->queue_context,
@@ -646,12 +663,12 @@ gearman_return_t gearman_server_queue_replay(gearman_server_st *server)
   if (server->queue_replay_fn == NULL)
     return GEARMAN_SUCCESS;
 
-  server->options|= GEARMAN_SERVER_QUEUE_REPLAY;
+  server->state.queue_startup= true;
 
   ret= (*(server->queue_replay_fn))(server, (void *)server->queue_context,
                                     _queue_replay_add, server);
 
-  server->options&= (gearman_server_options_t)~GEARMAN_SERVER_QUEUE_REPLAY;
+  server->state.queue_startup= false;
 
   return ret;
 }
@@ -662,7 +679,7 @@ void *gearman_server_queue_context(const gearman_server_st *server)
 }
 
 void gearman_server_set_queue_context(gearman_server_st *server,
-                                      const void *context)
+                                      void *context)
 {
   server->queue_context= context;
 }
@@ -739,7 +756,7 @@ static gearman_return_t _server_run_text(gearman_server_con_st *server_con,
   data= malloc(GEARMAN_TEXT_RESPONSE_SIZE);
   if (data == NULL)
   {
-    GEARMAN_ERROR_SET(packet->gearman, "_server_run_text", "malloc")
+    gearman_log_error(packet->universal, "_server_run_text", "malloc");
     return GEARMAN_MEMORY_ALLOCATION_FAILURE;
   }
   total= GEARMAN_TEXT_RESPONSE_SIZE;
@@ -756,7 +773,7 @@ static gearman_return_t _server_run_text(gearman_server_con_st *server_con,
     for (thread= server_con->thread->server->thread_list; thread != NULL;
          thread= thread->next)
     {
-      GEARMAN_SERVER_THREAD_LOCK(thread)
+      (void) pthread_mutex_lock(&thread->lock);
 
       for (con= thread->con_list; con != NULL; con= con->next)
       {
@@ -772,9 +789,9 @@ static gearman_return_t _server_run_text(gearman_server_con_st *server_con,
           new_data= realloc(data, total + GEARMAN_TEXT_RESPONSE_SIZE);
           if (new_data == NULL)
           {
-            GEARMAN_SERVER_THREAD_UNLOCK(thread)
+            (void) pthread_mutex_unlock(&thread->lock);
             free(data);
-            GEARMAN_ERROR_SET(packet->gearman, "_server_run_text", "malloc")
+            gearman_log_error(packet->universal, "_server_run_text", "malloc");
             return GEARMAN_MEMORY_ALLOCATION_FAILURE;
           }
 
@@ -802,7 +819,7 @@ static gearman_return_t _server_run_text(gearman_server_con_st *server_con,
         size+= (size_t)snprintf(data + size, total - size, "\n");
       }
 
-      GEARMAN_SERVER_THREAD_UNLOCK(thread)
+      (void) pthread_mutex_unlock(&thread->lock);
     }
 
     if (size < total)
@@ -821,7 +838,7 @@ static gearman_return_t _server_run_text(gearman_server_con_st *server_con,
         if (new_data == NULL)
         {
           free(data);
-          GEARMAN_ERROR_SET(packet->gearman, "_server_run_text", "malloc")
+          gearman_log_error(packet->universal, "_server_run_text", "malloc");
           return GEARMAN_MEMORY_ALLOCATION_FAILURE;
         }
 
@@ -916,14 +933,15 @@ static gearman_return_t _server_run_text(gearman_server_con_st *server_con,
 
   server_packet->packet.magic= GEARMAN_MAGIC_TEXT;
   server_packet->packet.command= GEARMAN_COMMAND_TEXT;
-  server_packet->packet.options|= (GEARMAN_PACKET_COMPLETE |
-                                   GEARMAN_PACKET_FREE_DATA);
+  server_packet->packet.options.complete= true;
+  server_packet->packet.options.free_data= true;
+
   server_packet->packet.data= data;
   server_packet->packet.data_size= strlen(data);
 
-  GEARMAN_SERVER_THREAD_LOCK(server_con->thread)
+  (void) pthread_mutex_lock(&server_con->thread->lock);
   GEARMAN_FIFO_ADD(server_con->io_packet, server_packet,)
-  GEARMAN_SERVER_THREAD_UNLOCK(server_con->thread)
+  (void) pthread_mutex_unlock(&server_con->thread->lock);
 
   gearman_server_con_io_add(server_con);
 
@@ -949,18 +967,18 @@ _server_queue_work_data(gearman_server_job_st *server_job,
 
     if (packet->data_size > 0)
     {
-      if (packet->options & GEARMAN_PACKET_FREE_DATA &&
+      if (packet->options.free_data &&
           server_client->job_next == NULL)
       {
         data= (uint8_t *)(packet->data);
-        packet->options&= (gearman_packet_options_t)~GEARMAN_PACKET_FREE_DATA;
+        packet->options.free_data= false;
       }
       else
       {
         data= malloc(packet->data_size);
         if (data == NULL)
         {
-          GEARMAN_ERROR_SET(packet->gearman, "_server_run_command", "malloc")
+          gearman_log_error(packet->universal, "_server_run_command", "malloc");
           return GEARMAN_MEMORY_ALLOCATION_FAILURE;
         }
 
